@@ -280,11 +280,14 @@ Efficient use of soft deletes:
 
 import itertools
 import logging
+import os
 import re
 import time
 
-from oslo.utils import timeutils
+from oslo_utils import timeutils
 import six
+from sqlalchemy import event
+from sqlalchemy import exc
 import sqlalchemy.orm
 from sqlalchemy import pool
 from sqlalchemy.sql.expression import literal_column
@@ -293,8 +296,8 @@ from sqlalchemy.sql.expression import select
 from oslo_db._i18n import _LW
 from oslo_db import exception
 from oslo_db import options
-from oslo_db.sqlalchemy import compat
 from oslo_db.sqlalchemy import exc_filters
+from oslo_db.sqlalchemy import update_match
 from oslo_db.sqlalchemy import utils
 
 LOG = logging.getLogger(__name__)
@@ -405,10 +408,14 @@ def create_engine(sql_connection, sqlite_fk=False, mysql_sql_mode=None,
     exc_filters.register_engine(engine)
 
     # register engine connect handler
-    compat.engine_connect(engine, _connect_ping_listener)
+    event.listen(engine, "engine_connect", _connect_ping_listener)
 
     # initial connect + test
-    _test_connection(engine, max_retries, retry_interval)
+    # NOTE(viktors): the current implementation of _test_connection()
+    #                does nothing, if max_retries == 0, so we can skip it
+    if max_retries:
+        test_conn = _test_connection(engine, max_retries, retry_interval)
+        test_conn.close()
 
     return engine
 
@@ -476,6 +483,8 @@ def _init_connection_args(url, engine_args, **kw):
 def _init_events(engine, thread_checkin=True, connection_trace=False, **kw):
     """Set up event listeners for all database backends."""
 
+    _add_process_guards(engine)
+
     if connection_trace:
         _add_trace_comments(engine)
 
@@ -576,7 +585,7 @@ def _test_connection(engine, max_retries, retry_interval):
     de_ref = None
     for attempt in attempts:
         try:
-            return exc_filters.handle_connect_error(engine)
+            return engine.connect()
         except exception.DBConnectionError as de:
             msg = _LW('SQL connection failed. %s attempts left.')
             LOG.warning(msg, max_retries - attempt)
@@ -595,6 +604,27 @@ class Query(sqlalchemy.orm.query.Query):
                             'deleted_at': timeutils.utcnow()},
                            synchronize_session=synchronize_session)
 
+    def update_returning_pk(self, values, surrogate_key):
+        """Perform an UPDATE, returning the primary key of the matched row.
+
+        This is a method-version of
+        oslo_db.sqlalchemy.update_match.update_returning_pk(); see that
+        function for usage details.
+
+        """
+        return update_match.update_returning_pk(self, values, surrogate_key)
+
+    def update_on_match(self, specimen, surrogate_key, values, **kw):
+        """Emit an UPDATE statement matching the given specimen.
+
+        This is a method-version of
+        oslo_db.sqlalchemy.update_match.update_on_match(); see that function
+        for usage details.
+
+        """
+        return update_match.update_on_match(
+            self, specimen, surrogate_key, values, **kw)
+
 
 class Session(sqlalchemy.orm.session.Session):
     """Custom Session class to avoid SqlAlchemy Session monkey patching."""
@@ -609,6 +639,35 @@ def get_maker(engine, autocommit=True, expire_on_commit=False):
                                        query_cls=Query)
 
 
+def _add_process_guards(engine):
+    """Add multiprocessing guards.
+
+    Forces a connection to be reconnected if it is detected
+    as having been shared to a sub-process.
+
+    """
+
+    @sqlalchemy.event.listens_for(engine, "connect")
+    def connect(dbapi_connection, connection_record):
+        connection_record.info['pid'] = os.getpid()
+
+    @sqlalchemy.event.listens_for(engine, "checkout")
+    def checkout(dbapi_connection, connection_record, connection_proxy):
+        pid = os.getpid()
+        if connection_record.info['pid'] != pid:
+            LOG.debug(_LW(
+                "Parent process %(orig)s forked (%(newproc)s) with an open "
+                "database connection, "
+                "which is being discarded and recreated."),
+                {"newproc": pid, "orig": connection_record.info['pid']})
+            connection_record.connection = connection_proxy.connection = None
+            raise exc.DisconnectionError(
+                "Connection record belongs to pid %s, "
+                "attempting to check out in pid %s" %
+                (connection_record.info['pid'], pid)
+            )
+
+
 def _add_trace_comments(engine):
     """Add trace comments.
 
