@@ -17,10 +17,11 @@
 #    under the License.
 
 import collections
+import contextlib
 import logging
 import re
 
-from oslo.utils import timeutils
+from oslo_utils import timeutils
 import six
 import sqlalchemy
 from sqlalchemy import Boolean
@@ -29,14 +30,12 @@ from sqlalchemy import Column
 from sqlalchemy.engine import Connectable
 from sqlalchemy.engine import reflection
 from sqlalchemy.engine import url as sa_url
-from sqlalchemy.ext.compiler import compiles
 from sqlalchemy import func
 from sqlalchemy import Index
 from sqlalchemy import inspect
 from sqlalchemy import Integer
 from sqlalchemy import MetaData
 from sqlalchemy.sql.expression import literal_column
-from sqlalchemy.sql.expression import UpdateBase
 from sqlalchemy.sql import text
 from sqlalchemy import String
 from sqlalchemy import Table
@@ -148,10 +147,13 @@ def paginate_query(query, model, limit, sort_keys, marker=None,
             raise ValueError(_("Unknown sort direction, "
                                "must be 'desc' or 'asc'"))
         try:
-            sort_key_attr = inspect(model).\
+            inspect(model).\
                 all_orm_descriptors[current_sort_key]
         except KeyError:
             raise exception.InvalidSortKey()
+        else:
+            sort_key_attr = getattr(model, current_sort_key)
+
         query = query.order_by(sort_dir_func(sort_key_attr))
 
     # Add pagination
@@ -187,6 +189,18 @@ def paginate_query(query, model, limit, sort_keys, marker=None,
     return query
 
 
+def to_list(x, default=None):
+    if x is None:
+        return default
+    if not isinstance(x, collections.Iterable) or \
+            isinstance(x, six.string_types):
+        return [x]
+    elif isinstance(x, list):
+        return x
+    else:
+        return list(x)
+
+
 def _read_deleted_filter(query, db_model, deleted):
     if 'deleted' not in db_model.__table__.columns:
         raise ValueError(_("There is no `deleted` column in `%s` table. "
@@ -345,19 +359,40 @@ def get_table(engine, name):
     return Table(name, metadata, autoload=True)
 
 
-class InsertFromSelect(UpdateBase):
-    """Form the base for `INSERT INTO table (SELECT ... )` statement."""
-    def __init__(self, table, select):
-        self.table = table
-        self.select = select
+class InsertFromSelect(object):
+    """Form the base for `INSERT INTO table (SELECT ... )` statement.
 
+    DEPRECATED: this class is deprecated and will be removed from oslo.db
+    in a few releases. Use default SQLAlchemy insert from select implementation
+    instead
 
-@compiles(InsertFromSelect)
-def visit_insert_from_select(element, compiler, **kw):
-    """Form the `INSERT INTO table (SELECT ... )` statement."""
-    return "INSERT INTO %s %s" % (
-        compiler.process(element.table, asfrom=True),
-        compiler.process(element.select))
+    :param table: table to insert records
+    :param select: select query
+    :param cols: list of columns to specify in insert clause
+    :return: SQLAlchemy :class:`Insert` object instance
+
+    Usage:
+
+    .. code-block:: python
+
+      select = sql.select(table_from)
+      insert = InsertFromSelect(table_to, select,
+                                ['id', 'name', 'insert_date'])
+      engine.execute(insert)
+
+    """
+    # NOTE(tdurakov): Insert from select implementation added to SQLAlchemy
+    # starting from version 0.8.7. Default SQLAlchemy implementation should be
+    # used instead of this. Deprecated.
+
+    def __new__(cls, table, select, cols=None):
+        if not cols:
+            cols = [c.name for c in table.c]
+
+        return table.insert(inline=True).from_select(cols, select)
+
+    def __init__(self, table, select, cols=None):
+        pass
 
 
 def _get_not_supported_column(col_name_col_instance, column_name):
@@ -1012,3 +1047,126 @@ def get_non_innodb_tables(connectable, skip_tables=('migrate_version',
     query = text(query_str)
     noninnodb = connectable.execute(query, **params)
     return [i[0] for i in noninnodb]
+
+
+class NonCommittingConnectable(object):
+    """A ``Connectable`` substitute which rolls all operations back.
+
+    ``NonCommittingConnectable`` forms the basis of mock
+    ``Engine`` and ``Connection`` objects within a test.   It provides
+    only that part of the API that should reasonably be used within
+    a single-connection test environment (e.g. no engine.dispose(),
+    connection.invalidate(), etc. ).   The connection runs both within
+    a transaction as well as a savepoint.   The transaction is there
+    so that any operations upon the connection can be rolled back.
+    If the test calls begin(), a "pseduo" transaction is returned that
+    won't actually commit anything.   The subtransaction is there to allow
+    a test to successfully call rollback(), however, where all operations
+    to that point will be rolled back and the operations can continue,
+    simulating a real rollback while still remaining within a transaction
+    external to the test.
+
+    """
+
+    def __init__(self, connection):
+        self.connection = connection
+        self._trans = connection.begin()
+        self._restart_nested()
+
+    def _restart_nested(self):
+        self._nested_trans = self.connection.begin_nested()
+
+    def _dispose(self):
+        if not self.connection.closed:
+            self._nested_trans.rollback()
+            self._trans.rollback()
+            self.connection.close()
+
+    def execute(self, obj, *multiparams, **params):
+        """Executes the given construct and returns a :class:`.ResultProxy`."""
+
+        return self.connection.execute(obj, *multiparams, **params)
+
+    def scalar(self, obj, *multiparams, **params):
+        """Executes and returns the first column of the first row."""
+
+        return self.connection.scalar(obj, *multiparams, **params)
+
+
+class NonCommittingEngine(NonCommittingConnectable):
+    """``Engine`` -specific non committing connectbale."""
+
+    @property
+    def url(self):
+        return self.connection.engine.url
+
+    @property
+    def engine(self):
+        return self
+
+    def connect(self):
+        return NonCommittingConnection(self.connection)
+
+    @contextlib.contextmanager
+    def begin(self):
+        conn = self.connect()
+        trans = conn.begin()
+        try:
+            yield conn
+        except Exception:
+            trans.rollback()
+        else:
+            trans.commit()
+
+
+class NonCommittingConnection(NonCommittingConnectable):
+    """``Connection`` -specific non committing connectbale."""
+
+    def close(self):
+        """Close the 'Connection'.
+
+        In this context, close() is a no-op.
+
+        """
+        pass
+
+    def begin(self):
+        return NonCommittingTransaction(self, self.connection.begin())
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, *arg):
+        pass
+
+
+class NonCommittingTransaction(object):
+    """A wrapper for ``Transaction``.
+
+    This is to accommodate being able to guaranteed start a new
+    SAVEPOINT when a transaction is rolled back.
+
+    """
+    def __init__(self, provisioned, transaction):
+        self.provisioned = provisioned
+        self.transaction = transaction
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type, value, traceback):
+        if type is None:
+            try:
+                self.commit()
+            except Exception:
+                self.rollback()
+                raise
+        else:
+            self.rollback()
+
+    def commit(self):
+        self.transaction.commit()
+
+    def rollback(self):
+        self.transaction.rollback()
+        self.provisioned._restart_nested()
