@@ -21,7 +21,7 @@ import logging
 
 import fixtures
 import mock
-from oslo.config import cfg
+from oslo_config import cfg
 from oslotest import base as oslo_test
 import sqlalchemy
 from sqlalchemy import Column, MetaData, Table
@@ -167,21 +167,21 @@ class SQLiteSavepointTest(test_base.DbTestCase):
         )
 
 
-class FakeDBAPIConnection():
+class FakeDBAPIConnection(object):
     def cursor(self):
         return FakeCursor()
 
 
-class FakeCursor():
+class FakeCursor(object):
     def execute(self, sql):
         pass
 
 
-class FakeConnectionProxy():
+class FakeConnectionProxy(object):
     pass
 
 
-class FakeConnectionRec():
+class FakeConnectionRec(object):
     pass
 
 
@@ -195,7 +195,7 @@ class ProgrammingError(Exception):
 
 class FakeDB2Engine(object):
 
-    class Dialect():
+    class Dialect(object):
 
         def is_disconnect(self, e, *args):
             expected_error = ('SQL30081N: DB2 Server connection is no longer '
@@ -229,20 +229,22 @@ class MySQLModeTestCase(test_base.MySQLOpportunisticTestCase):
 
     def setUp(self):
         super(MySQLModeTestCase, self).setUp()
-
-        self.engine = session.create_engine(self.engine.url,
-                                            mysql_sql_mode=self.mysql_mode)
-        self.connection = self.engine.connect()
+        mode_engine = session.create_engine(
+            self.engine.url,
+            mysql_sql_mode=self.mysql_mode)
+        self.connection = mode_engine.connect()
 
         meta = MetaData()
-        meta.bind = self.engine
         self.test_table = Table(_TABLE_NAME + "mode", meta,
                                 Column('id', Integer, primary_key=True),
                                 Column('bar', String(255)))
-        self.test_table.create()
+        self.test_table.create(self.connection)
 
-        self.addCleanup(self.test_table.drop)
-        self.addCleanup(self.connection.close)
+        def cleanup():
+            self.test_table.drop(self.connection)
+            self.connection.close()
+            mode_engine.dispose()
+        self.addCleanup(cleanup)
 
     def _test_string_too_long(self, value):
         with self.connection.begin():
@@ -646,6 +648,34 @@ class CreateEngineTest(oslo_test.BaseTestCase):
         )
 
 
+class ProcessGuardTest(test_base.DbTestCase):
+    def test_process_guard(self):
+        self.engine.dispose()
+
+        def get_parent_pid():
+            return 4
+
+        def get_child_pid():
+            return 5
+
+        with mock.patch("os.getpid", get_parent_pid):
+            with self.engine.connect() as conn:
+                dbapi_id = id(conn.connection.connection)
+
+        with mock.patch("os.getpid", get_child_pid):
+            with self.engine.connect() as conn:
+                new_dbapi_id = id(conn.connection.connection)
+
+        self.assertNotEqual(dbapi_id, new_dbapi_id)
+
+        # ensure it doesn't trip again
+        with mock.patch("os.getpid", get_child_pid):
+            with self.engine.connect() as conn:
+                newer_dbapi_id = id(conn.connection.connection)
+
+        self.assertEqual(new_dbapi_id, newer_dbapi_id)
+
+
 class PatchStacktraceTest(test_base.DbTestCase):
 
     def test_trace(self):
@@ -665,9 +695,11 @@ class PatchStacktraceTest(test_base.DbTestCase):
 
             session._add_trace_comments(engine)
             conn = engine.connect()
+            orig_do_exec = engine.dialect.do_execute
             with mock.patch.object(engine.dialect, "do_execute") as mock_exec:
 
-                conn.execute("select * from table")
+                mock_exec.side_effect = orig_do_exec
+                conn.execute("select 1;")
 
             call = mock_exec.mock_calls[0]
 
